//---------------------------------------------------------------------
//STARTHEADER
//
//ver: 1.4
//engineReq: 6.5
//date: 060902
//
/////////////////////////////////////////
//title: Material Effects Library
// Conitec / jcl 2006
/////////////////////////////////////////
//class: ENVIR
//type: USER
//
//help: Standard Material Effects for Terrain, Water, Entities.
/// The effects are automatically activated by assigning the action.
/// Requires A6.50 Commercial or above, VS 1.1, PS 1.3, default.fx
//
//needs: default.fx
//prefix: fx_
//idcode: 601
//
//ENDHEADER
// ---------------------------------------------------------------------

ifndef mtlFX;
define mtlFX;

STRING fx_needs = "default.fx";	// for the WED resource window

///////////////////////////////////////////////////////////////////////
// helper functions
MATERIAL* mtl_temp;

//function: mtl_copy
//desc: helper function for copying predefined material properties 
//desc: to the mtl pointer.
//
function mtl_copy(mtl_source)
{
	mtl_temp = mtl_source;
	vec_set(mtl.ambient_blue,mtl_temp.ambient_blue);
	vec_set(mtl.specular_blue,mtl_temp.specular_blue);
	vec_set(mtl.diffuse_blue,mtl_temp.diffuse_blue);
	vec_set(mtl.emissive_blue,mtl_temp.emissive_blue);
	mtl.power = mtl_temp.power;
	mtl.albedo = mtl_temp.albedo;
}

///////////////////////////////////////////////////////////////////////
//section: Terrain

//desc: Terrain multitexture material. 
//desc: Use fx_terraintex3 to assign this material.
//
MATERIAL mtl_terraintex3
{
	effect = "

//enable: DirectX Lighting
//help: Use the DirectX Formula for dynamic lights.
//help: Otherwise use the Conitec Formula that produces smoother light.
//id: 11
//#define DXLIGHTING

//enable: DirectX Fog
//help: Use the DirectX Formula for z-based fog.
//help: Otherwise use the Conitec Formula that produces distance-based fog.
//id: 12
#define DXFOG

//enable: Skin4 Masked on Green
//help: Skin 4 contains a texture masked by the Skin1 green channel.
//id: 13
#define GREENMASK

//enable: Shadow Map on Blue
//help: Terrain contains a shadow map on the Skin1 blue channel.
//help: With PS 1.1 (GeForce3), sun & dynamic lights are then disabled.
//id: 14
#define SHADOWMAP

//enable: PS 1.3
//help: Support pixelshader version 1.3 if available.
//help: This allows shadow map and lights on terrain at the same time.
//id: 15
#define PS13

	#include <transform>
	#include <sun>
	#include <lights>
	#include <fog>
	#include <normal>

	Texture entSkin1; // Red/green for blending, blue for shadow
	Texture entSkin2; // Basic tiled terrain texture
	Texture entSkin3; // Red masked tiled texture
	Texture entSkin4; // Green masked tiled texture

	float4 vecSkill41;
	float fAmbient;

	sampler sMaskTex = sampler_state { Texture = <entSkin1>; };
	sampler sBaseTex = sampler_state { Texture = <entSkin2>; };
	sampler sRedTex = sampler_state { Texture = <entSkin3>; };
#ifdef GREENMASK
	sampler sGreenTex = sampler_state {	Texture = <entSkin4>; };
#endif

//////////////////////////////////////////////////////////////////////
	struct out_terraintex3 // Output to the pixelshader fragment
	{
		float4 Pos		: POSITION;
		float4 Color	: COLOR0;
		float  Fog		: FOG;
		float2 MaskCoord: TEXCOORD0;
		float2 BaseCoord: TEXCOORD1;
		float2 RedCoord : TEXCOORD2;
#ifdef GREENMASK
		float2 GreenCoord: TEXCOORD3;
#endif
	};
	
	out_terraintex3 vs_terraintex3(
		float4 inPos : POSITION,
		float3 inNormal : NORMAL,
		float2 inTexCoord0 : TEXCOORD0)
	{
		out_terraintex3 Out;

		Out.Pos = DoTransform(inPos); // transform to screen coordinates

// rotate and normalize the normal
		float3 N = DoNormal(inNormal);
		float3 P = mul(inPos,matWorld);

		Out.Color = fAmbient + DoSunLight(N); // Add ambient and sun light
		for (int i=0; i<6; i++)  // Add 6 dynamic lights (maximum for vs 1.1)
			Out.Color += DoLight(P,N,i);
		Out.Fog = DoFog(inPos); // Add fog

// scale the texture coordinates for the masked textures
		Out.MaskCoord = inTexCoord0.xy;
		Out.BaseCoord = inTexCoord0.xy * vecSkill41.x;
		Out.RedCoord = inTexCoord0.xy * vecSkill41.y;
#ifdef GREENMASK
		Out.GreenCoord = inTexCoord0.xy * vecSkill41.z;
#endif
		return Out;
	}

#ifdef PS13 // version 1.3 pixelshader
	float4 ps_terraintex3_13(out_terraintex3 In): COLOR
	{
// retrieve the pixels for the textures and the masks
		float4 MaskColor = tex2D(sMaskTex,In.MaskCoord);
		float4 BaseColor = tex2D(sBaseTex,In.BaseCoord);
		float4 RedColor = tex2D(sRedTex,In.RedCoord);

// blend the red and green textures over the base texture
#ifdef GREENMASK
		float4 GreenColor = tex2D(sGreenTex,In.GreenCoord);
		float4 BaseRed = lerp(BaseColor,RedColor,MaskColor.r);
		float4 FinalColor = lerp(BaseRed,GreenColor,MaskColor.g);
#else
		float4 FinalColor = lerp(BaseColor,RedColor,MaskColor.r);
#endif

// Add the vertex light and shadow map
#ifdef SHADOWMAP
//entry: Shadow Map Offset
//cntl: spin 0 1.0 0.05
//help: Light offset (0..1) to be subtracted from the shadow map
		float offset = 0.25;
		FinalColor *= In.Color + (MaskColor.b - offset);
#else
		FinalColor *= In.Color;
#endif
		FinalColor.a = 1.0f;	// prevent transparency
		return FinalColor;
	}
#endif

	float4 ps_terraintex3_11(out_terraintex3 In): COLOR
	{
// retrieve the pixels for the textures and the masks
		float4 MaskColor = tex2D(sMaskTex,In.MaskCoord);
		float4 BaseColor = tex2D(sBaseTex,In.BaseCoord);
		float4 RedColor = tex2D(sRedTex,In.RedCoord);

// blend the red and green textures over the base texture
#ifdef GREENMASK
		float4 GreenColor = tex2D(sGreenTex,In.GreenCoord);
		float4 BaseRed = lerp(BaseColor,RedColor,MaskColor.r);
		float4 FinalColor = lerp(BaseRed,GreenColor,MaskColor.g);
#else
		float4 FinalColor = lerp(BaseColor,RedColor,MaskColor.r);
#endif

// Add the vertex light or shadow map, plus the entity ambient
		FinalColor *= In.Color;
		FinalColor.a = 1.0f;	// prevent transparency
		return FinalColor;
	}


#ifdef PS13 // version 1.3 pixelshader
	technique terraintex3_13
	{
		pass one
		{
			sampler[0] = (sMaskTex);
			sampler[1] = (sBaseTex);
			sampler[2] = (sRedTex);
#ifdef GREENMASK
			sampler[3] = (sGreenTex);
#endif

			VertexShader = compile vs_1_1 vs_terraintex3();
			PixelShader = compile ps_1_3 ps_terraintex3_13();
		}
	}
#endif

// fallback for pixelshader 1.1
	technique terraintex3_11
	{
		pass one
		{
			sampler[0] = (sMaskTex);
			sampler[1] = (sBaseTex);
			sampler[2] = (sRedTex);
#ifdef GREENMASK
			sampler[3] = (sGreenTex);
#endif

			VertexShader = compile vs_1_1 vs_terraintex3();
			PixelShader = compile ps_1_1 ps_terraintex3_11();
		}
	}

// fallback if nothing works
	technique fallback { pass one { } }

";
}

//action: fx_terraintex3
//title: Multitexture Terrain 
//
//skill2: ScaleSkin2 15
//help: Skin2 base texture scale, 1..50, default 15.
//skill3: ScaleSkin3 15
//help: Skin3 red texture scale, 1..50, default 15.
//skill4: ScaleSkin4 15
//help: Skin4 green texture scale, 1..50, default 15.
//desc:
/// Blends 3 textures together, according to
/// the red and green channel in the first skin.
/// The blue channel can be used for a shadow map. 
/// Skin1 = Terrain RGB blend & shadow mask.
/// Skin2 = Base tiled terrain texture (grass).
/// Skin3 = Red channel masked tiled texture (sand).
/// Skin4 = Green channel masked tiled texture (rock).
/// Skin5 = Optional base texture for non-shader hardware.
/// Skin6 = Optional detail texture for non-shader hardware.
//
//Based on ideas by Thomas 'ventilator' Oppl
//and Eric 'Steempipe' Hendrickson-Lambert
//
action fx_terraintex3()
{	
// use skin 5 for non-shader hardware
	if (d3d_shaderversion < 1111) {
		if (ent_skins(my) >= 5)	{ my.skin = 5; }	
		return; 
	}

// if entity skins are missing, replace them by standard skins	
	mtl = mtl_terraintex3;
	mtl_copy(mat_terrain);
	my.material = mtl;
	if (ent_skins(my) < 4) { mtl.skin4 = bmap_create("rock.tga"); }
	if (ent_skins(my) < 3) { mtl.skin3 = bmap_create("sand.tga"); }
	if (ent_skins(my) < 2) { mtl.skin2 = bmap_create("grass.tga"); }

// copy the texture scales to vecSkill41
	if (my.skill2) { my.skill41 = float(my.skill2); }
	else { my.skill41 = float(15); }
	if (my.skill3) { my.skill42 = float(my.skill3); }
	else { 	my.skill42 = float(15); }
	if (my.skill4) { my.skill43 = float(my.skill4); }
	else { 	my.skill43 = float(15); }

	my.DYNAMIC = OFF;	// when all done, set terrain static
}

///////////////////////////////////////////////////////////////////////
// Section: Mirrors

//desc: mtl_water_mirror
//desc: Realtime water reflection with ripple effect.
//desc: Required: A6.42 Pro, VS 1.1, PS 1.1
//desc: Note: can look bad on old 3D cards (GF4 or below)
//desc: Use fx_water_mirror to assign this material
//
MATERIAL mtl_water_mirror
{
	effect=
	"
//enable: Mirror Fresnel effect
//help: Water transparency depends on view angle
//id: 21
#define MIRROR_FRESNEL

	#include <transform>
	#include <fog>
	#include <pos>
	#include <normal>
	
	float4 vecTime;
	float4 vecSkill41; // Wind_X, Wind_Y, Ripple, Scale
	float4 vecColor;	

	texture entSkin1;
	texture mtlSkin2;

	sampler sBumpTex = sampler_state
	{
		Texture = <entSkin1>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Wrap;
		Addressv = Wrap;
	};

	sampler sMirrorTex = sampler_state
	{
		Texture = <mtlSkin2>;
		MipFilter = Linear;
		MinFilter = Linear;
		MagFilter = Linear;
		AddressU = Clamp;
		Addressv = Clamp;
	};


	struct out_water_mirror // Output to the pixelshader fragment
	{
		float4 Pos	: POSITION;
		float4 Color: COLOR0;
		float  Fog	: FOG;
		float2 Tex0	: TEXCOORD0;
		float2 Tex1	: TEXCOORD1;
	};

	out_water_mirror vs_water_mirror
	(
		in float4 inPos		: POSITION,
		in float3 inNormal	: NORMAL,
		in float4 inTex0	: TEXCOORD0
	)
	{
		out_water_mirror Out;
		
		Out.Pos = DoTransform(inPos); // transform to screen coordinates
		Out.Fog = DoFog(inPos);

// bump and reflection coordinates
		Out.Tex0 = (inTex0 + vecSkill41.xy * vecTime.w) * vecSkill41.w;
		Out.Tex1 = 0.5 * (1.0 + (Out.Pos/Out.Pos.w));

// color and transparency
		Out.Color = vecColor;
#ifdef MIRROR_FRESNEL
		float4 P = DoPos(inPos); // vector world position
		float3 N = DoNormal(inNormal); // normal world orientation
		float4 vecToView = normalize(vecViewPos-P); // direction towards camera
		Out.Color.a = 0.5 + vecColor.a * (1.0 - dot(vecToView,N));
#endif		
		return Out;
	}
	
#ifdef WONTWORK
	float4 ps_mirror_straight(out_water_mirror In): COLOR
	{
		return tex2D(sMirrorTex,In.Tex1) * In.Color;
	}
	
	float4 ps_water_mirror(out_water_mirror In): COLOR
	{
		float4 Bump = tex2D(sBumpTex,In.Tex0);
		float2 Coord = In.Tex1 + Bump.xy * vecSkill41.z;
		return tex2D(sMirrorTex,Coord) * In.Color;
	}
#endif

//////////////////////////////////////////////////////////////////
	technique water_mirror
	{
		pass one
		{
			AlphaBlendEnable = True;

			sampler[0] = (sBumpTex);
			sampler[1] = (sMirrorTex);

			VertexShader = compile vs_1_1 vs_water_mirror();

// bump matrix - required for texbem
			BumpEnvMat00[1] = (vecSkill41.z);
			BumpEnvMat11[1] = (vecSkill41.z);
			BumpEnvMat01[1] = 0.0;
			BumpEnvMat10[1] = 0.0;

// We need an asm shader here - for some reason,
// ps_1_1 HLSL tex2D won't sample modified coordinates
			PixelShader = asm
			{
				ps_1_1
				tex t0         // sample Stage0 Texture rg 
				texbem t1,t0   // multiply rg with BumpEnvMat to uv, 
							   // and sample Stage1 Texture using uv+t1
				mul r0,v0,t1   // multiply Stage1 Texture by Color + Alpha
			};
		}
	}

	technique fallback { pass one { } }
	";
}

//////////////////////////////////////////////////////////////////

//enable: Mirror debugging
//help: Make mirror view visible by pressing F11
//id: 22
//define MIRROR_DEBUG;

ifdef MIRROR_DEBUG;
VIEW view_mirror { layer = 1; };
ifelse;
VIEW view_mirror { layer = -1; };	// render mirror view before camera view
endif;
BMAP* bmap_mirrortarget;
var mtlfx_mirrorvisible;

//Simulate a horizontal mirror by generating
//a camera view from below through the floor
function fx_mirror()
{
// do nothing if mirror is already running
	if (bmap_mirrortarget) { return; }

	bmap_mirrortarget = bmap_createblack(256,256,888);
	mtl_water_mirror.skin2 = bmap_mirrortarget;
	view_mirror.bmap = bmap_mirrortarget;
	view_mirror.size_x = bmap_width(view_mirror.bmap);
	view_mirror.size_y = bmap_height(view_mirror.bmap);

	vec_set(view_mirror.pnormal_x,vector(0,0,1.0));	// reflect upwards
	view_mirror.noshadow = on;	// suppress shadows in the mirror
	view_mirror.nocull = on;    // view through walls
	view_mirror.portalclip = on; // clip at portal plane

//enable: Suppress FLAG1 in mirror
//help: Don't display FLAG1 objects in the mirror views
//id: 23
	view_mirror.noflag1 = 1;	// suppress all entities with flag1 set

//enable: Suppress particles in mirror
//help: Don't display particles in the mirror views
//id: 24
	view_mirror.noparticle = 1;

//enable: Suppress shaders in mirror
//help: Don't render material effects in the mirror views
//id: 25
	view_mirror.noshader = 1;

	while (bmap_mirrortarget)
	{
		proc_late();	// camera must be moved and mtlfx_mirrorvisible set before
		if (mtlfx_mirrorvisible) 
		{ 
			view_mirror.VISIBLE = ON;
			view_mirror.genius = camera.genius;
			view_mirror.aspect = screen_size.x/screen_size.y; // screen aspect, independent of render target
			view_mirror.arc    = camera.arc;
			view_mirror.fog_start = camera.fog_start;
			view_mirror.fog_end   = camera.fog_end;
			view_mirror.clip_far  = camera.clip_far * 0.5;
			view_mirror.clip_near = camera.clip_near * 2;
			view_mirror.x 	   = camera.x;
			view_mirror.y 	   = camera.y;
			view_mirror.z 	   = 2*view_mirror.portal_z-camera.z;	// move the camera at its mirror position
			view_mirror.pan    = camera.pan;
			view_mirror.tilt   = -camera.tilt;	// flip the vertical camera angle
			view_mirror.roll   = -camera.roll;
		}
		else { 
// switch rendering off when all mirror objects are outside the frustrum
			view_mirror.VISIBLE = OFF; 
		}
		mtlfx_mirrorvisible = 0;

ifdef MIRROR_DEBUG;
		if (key_f11) {	// debugging - make mirror visible onscreen
			view_mirror.bmap = NULL;
		} else {
			view_mirror.bmap = bmap_mirrortarget;
		}	
endif;
		wait(1);
	}
}

//action: fx_water_mirror
//title: Realtime water reflection
//
//skill1: WindSpeed_X 2.0
//help: Motion in X direction, default 2
//skill2: WindSpeed_Y 2.0
//help: Motion in Y direction, default 2
//skill3: Ripple 0.5
//help: Ripple strength, default 0.5
//skill4: Scale  0.5
//help: Ripple scale, default 0.5
//skill5: Blue   240
//help: Water blue tint, 0..255
//skill6: Green  200
//help: Water green tint, 0..255
//skill7: Red    200
//help: Water red tint, 0..255
//desc: 
/// Realtime water reflection with ripple effect.
/// To be assigned to a water terrain, like water.hmp.
/// Skin1 contains a bump map for the water ripples.
/// Transparency is set through Albedo.
/// Required: A6.50+ Pro, VS 1.1, PS 1.1
/// Note: can look bad on 3D cards w/o clip planes (GF4 or below)
//
action fx_water_mirror()
{
	my.material = mtl_water_mirror;
	my.TRANSPARENT = ON;
	bmap_to_uv(bmap_for_entity(my,0));
	fx_mirror();	// activate mirror view
	
// copy shader parameters to vecSkill41	
	if (my.skill1) { my.skill41 = floatd(my.skill1,10000); }
	else { my.skill41 = floatd(2,10000); }
	if (my.skill2) { my.skill42 = floatd(my.skill2,10000); }
	else { my.skill42 = floatd(2,10000); }
	if (my.skill3) { my.skill43 = float(my.skill3*0.1); }
	else { my.skill43 = float(0.05); }
	if (my.skill4) { my.skill44 = float(my.skill4); }
	else { my.skill44 = float(0.5); }
	
	if (my.skill5) { 
		vec_set(my.blue,vector(my.skill5,my.skill6,my.skill7)); }
	else { 
		vec_set(my.blue,vector(240,200,200)); }
	
// update mirror when not clipped away 
	while(1) 
	{
		if (0 == my.clipped) 
		{
			vec_for_max(view_mirror.portal_x,my);
			vec_add(view_mirror.portal_x,my.x);
			mtlfx_mirrorvisible = 1;
		}
		wait(1);
	}
}
////////////////////////////////////////////////////////////////////

BMAP* bmap_skycube;

//desc: mtl_water_env
//desc: Environment reflection with ripple effect.
//desc: Required: A6.42+ Commercial, VS 1.1, PS 1.1
//desc: Use fx_water_env to assign this material
//
MATERIAL mtl_water_env
{
	flags = TANGENT;
	effect=
	"
//enable: Enviroment Fresnel effect
//help: Water transparency depends on view angle
//id: 31
#define ENVIRO_FRESNEL

	#include <transform>
	#include <fog>
	#include <pos>
	#include <normal>
	
	float4 vecTime;
	float4 vecSkill41; // Wind_X, Wind_Y, Ripple, Scale
	float4 vecColor;	

	texture entSkin1;
	texture mtlSkin2;

	sampler sBumpTex = sampler_state { Texture = (entSkin1); };
	sampler sCubeTex = sampler_state { Texture = (mtlSkin2); };

	struct out_water_env // Output to the pixelshader
	{
		float4 Pos	: POSITION;
		float4 Color: COLOR0;
		float  Fog	: FOG;
		float2 Tex0	: TEXCOORD0;
		float4 TanToCube0:	TEXCOORD1;
		float4 TanToCube1:	TEXCOORD2;
		float4 TanToCube2:	TEXCOORD3;
	};

	out_water_env vs_water_env
	(
		in float4 inPos		: POSITION,
		in float3 inNormal	: NORMAL,
		in float2 inTex0	: TEXCOORD0,
	    in float3 inTangent	: TEXCOORD2
	)
	{
		out_water_env Out;
		
		Out.Pos = DoTransform(inPos); // transform to screen coordinates
		Out.Fog = DoFog(inPos);

// bump and reflection coordinates
		Out.Tex0 = (inTex0 + vecSkill41.xy * vecTime.w) * vecSkill41.w;

		float3x3 ObjToTan;
		
		ObjToTan[0] = vecSkill41.z * inTangent;
		ObjToTan[1] = vecSkill41.z * cross(inTangent, inNormal);
		ObjToTan[2] = inNormal;

		Out.TanToCube0.xyz = mul(ObjToTan,matWorld[0].xyz);
		Out.TanToCube1.xyz = mul(ObjToTan,matWorld[1].xyz);
		Out.TanToCube2.xyz = mul(ObjToTan,matWorld[2].xyz);

		float4 P = DoPos(inPos); // vector world position
		float4 vecToView = normalize(vecViewPos-P); // direction towards camera

		Out.TanToCube0.w = vecToView.x;
		Out.TanToCube1.w = vecToView.y;
		Out.TanToCube2.w = vecToView.z;

// color and transparency
		Out.Color = vecColor;
#ifdef ENVIRO_FRESNEL
		float3 N = DoNormal(inNormal); // normal world orientation
		Out.Color.a = 0.5 + vecColor.a * (1.0 - dot(vecToView,N));
#endif		
		return Out;
	}
	
//////////////////////////////////////////////////////////////////
	technique water_env
	{
		pass one
		{
			AlphaBlendEnable = True;

			sampler[0] = (sBumpTex);
			sampler[3] = (sCubeTex);

			VertexShader = compile vs_1_1 vs_water_env();

			PixelShader = asm
			{
				ps_1_1
				tex t0		// Normalmap
				texm3x3pad   t1,t0_bx2	// TanToCube0
				texm3x3pad   t2,t0_bx2	// TanToCube1
				texm3x3vspec t3,t0_bx2	// TanToCube2, lookup in CubeTex
				mul r0,v0,t3   // multiply Stage3 Texture by Color + Alpha
			};			
		}
	}

	technique fallback { pass one { } }
	";
}

//action: fx_water_env
//title: Environment water reflection
//desc: 
//desc: Environment reflection with ripple effect.
/// To be assigned to a water terrain, like water.hmp.
/// Skin1 contains a bump map for the water ripples.
/// If the level map contains a sky cube, it is used 
/// for the reflection, otherwise a default sky is used.
/// Transparency is set through Albedo.
/// Required: A6.50+ Pro, VS 1.1, PS 1.1
//
//skill1: WindSpeed_X 2.0
//help: Motion in X direction, default 2
//skill2: WindSpeed_Y 2.0
//help: Motion in Y direction, default 2
//skill3: Ripple 0.5
//help: Ripple strength, default 0.5
//skill4: Scale  0.5
//help: Ripple scale, default 0.5
//skill5: Blue   240
//help: Water blue tint, 0..255
//skill6: Green  200
//help: Water green tint, 0..255
//skill7: Red    200
//help: Water red tint, 0..255
//
// Based on a concept by Eric 'Steempipe' Hendrickson-Lambert
//
action fx_water_env()
{
	my.material = mtl_water_env;
	my.TRANSPARENT = ON;
	bmap_to_normals(bmap_for_entity(my,0),1.0);
	if (!bmap_skycube) {
		if (!sky_cube_level) {
//entry: Default Environment Sky Cube
//help: This sky is used when the level map does not contain a sky cube 	
//id: 32
			bmap_skycube = bmap_create("skycube1+6.tga");
		} else {
			bmap_skycube = bmap_createblack(1536,256,24);
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,1),vector(0,0,0),vector(256,256,0));
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,2),vector(256,0,0),vector(256,256,0));
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,3),vector(512,0,0),vector(256,256,0));
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,4),vector(768,0,0),vector(256,256,0));
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,5),vector(1024,0,0),vector(256,256,0));
			bmap_blit(bmap_skycube,bmap_for_entity(sky_cube_level,6),vector(1280,0,0),vector(256,256,0));
		}
		bmap_to_cubemap(bmap_skycube);
	}
	mtl_water_env.skin2 = bmap_skycube;
	
// copy shader parameters to vecSkill41	
	if (my.skill1) { my.skill41 = floatd(my.skill1,10000); }
	else { my.skill41 = floatd(2,10000); }
	if (my.skill2) { my.skill42 = floatd(my.skill2,10000); }
	else { my.skill42 = floatd(2,10000); }

	if (my.skill3) { my.skill43 = float(my.skill3*0.5); }
	else { my.skill43 = float(0.25); }
	if (my.skill4) { my.skill44 = float(my.skill4*3); }
	else { my.skill44 = float(1.5); }
	
	if (my.skill5) { 
		vec_set(my.blue,vector(my.skill5,my.skill6,my.skill7)); }
	else { 
		vec_set(my.blue,vector(240,200,200)); }
}

////////////////////////////////////////////////////////////////////

function mtl_uvspeed_render()
{
// using the material matrix:
// u' = u * mtl.matrix11 + v * mtl.matrix21 + mtl.matrix31;
// v' = u * mtl.matrix12 + v * mtl.matrix22 + mtl.matrix32;
	mtl.matrix31 = floatd(my.skill1*total_ticks,256);
	mtl.matrix32 = floatd(my.skill2*total_ticks,256);
}

function mtl_uvspeed_init()
{
// copy standard model material properties
	mtl_copy(mat_model);
	mat_identity(mtl.matrix);
	mtl.event = mtl_uvspeed_render;
	mtl.ENABLE_RENDER = on;
}

MATERIAL mtl_uvspeed =
{
	event = mtl_uvspeed_init;
	effect = "
	matrix matMtl;

	technique uvspeed
	{
		pass one
		{
			TextureTransformFlags[0] = Count2;
			TextureTransform[0] = <matMtl>;
		}
	}
	technique fallback { pass one { } }
	";
}

//action: fx_uvspeed
//title: Texture shifting in u and v direction
//
//skill1: Speed_U 0
//help: Texture Speed in U direction, default 0
//skill2: Speed_V -2
//help: Texture Speed in V direction, default -2
//desc: 
//desc: Shifts a texture in u and v direction.
//desc: To be assigned to a model. 
//
action fx_uvspeed()
{
	my.material = mtl_uvspeed;
	if (0 == my.skill1 && 0 == my.skill2) { my.skill2 = -2; }
}

////////////////////////////////////////////////////////////////////
//section: General

//enable: Automaterial
//help: Automatically assign materials to textures with the same name.
//id: 91
DEFINE AUTOMATERIAL;
IFDEF AUTOMATERIAL;
var d3d_automaterial = 1;
IFELSE;
var d3d_automaterial = 0;
ENDIF;

// test material for testing if material assignment works
MATERIAL mtl_test { emissive_red = 255; }

////////////////////////////////////////////////////////////////////
endif;	// mtlFX
